@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 if (!CI) {
   // we're not being included by foxyproxy.js
@@ -297,7 +297,7 @@ var pacUtils =
 "}\n"+
 
 "function isInNet(ipaddr, pattern, maskstr) {\n"+
-"    var test = /^(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})$/(ipaddr);\n"+
+"    var test = /^(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})$/.exec(ipaddr);\n"+
 "    if (test == null) {\n"+
 "        ipaddr = dnsResolve(ipaddr);\n"+
 "        if (ipaddr == null)\n"+
