@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 // common fcns used throughout foxyproxy, exposed a an xpcom service
 
@@ -14,11 +14,13 @@ const CI = Components.interfaces;
 const CC = Components.classes;
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
+var fp; 
+
 function Common() {
   this.wrappedJSObject = this;
-  uuid = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject.
-    isFoxyProxySimple() ? "foxyproxy-basic@eric.h.jung" : "foxyproxy@eric.h.jung";
-
+  this.appInfo = CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULAppInfo);
+  fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;   
+  let uuid = fp.isFoxyProxySimple() ? "foxyproxy-basic@eric.h.jung" : "foxyproxy@eric.h.jung";
   // Get installed version
   if ("@mozilla.org/extensions/manager;1" in CC) {
     // Pre-Gecko 2.0
@@ -51,16 +53,11 @@ Common.prototype = {
     var wm = CC["@mozilla.org/appshell/window-mediator;1"].getService(CI.nsIWindowMediator);
     // The next line always returns an object, even if the enum has no elements, so we can't use it to determine between applications
     //var e = wm.getEnumerator("navigator:browser") || wm.getEnumerator("Songbird:Main")
-    return wm.getMostRecentWindow("navigator:browser") ? wm.getEnumerator("navigator:browser") : (wm.getEnumerator("Songbird:Main") ? wm.getEnumerator("Songbird:Main") : wm.getEnumerator("mail:3pane"));
+    return wm.getMostRecentWindow("navigator:browser") ? wm.getEnumerator("navigator:browser") : (wm.getMostRecentWindow("Songbird:Main") ? wm.getEnumerator("Songbird:Main") : wm.getEnumerator("mail:3pane"));
   },
 
   openAndReuseOneTabPerURL : function(aURL) {
-    var wm = CC["@mozilla.org/appshell/window-mediator;1"].getService(CI.nsIWindowMediator);
-    var winEnum = wm.getEnumerator("navigator:browser");
-    if (!winEnum.hasMoreElements())
-      winEnum = wm.getEnumerator("Songbird:Main");
-    if (!winEnum.hasMoreElements())
-      winEnm = wm.getEnumerator("mail:3pane");
+    var winEnum = this.getEnumerator();
     while (winEnum.hasMoreElements()) {
       var win = winEnum.getNext();
       var browser = win.getBrowser();
@@ -97,7 +94,7 @@ Common.prototype = {
   },
   
   validatePattern : function(win, isRegEx, p) {
-    var origPat = p, fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;
+    var origPat = p; 
     p = p.replace(/^\s*|\s*$/g,"");
     if (p == "") {
       fp.alert(win, fp.getMessage("pattern.required"));
@@ -132,7 +129,6 @@ Common.prototype = {
     var doc = args.document || document;
     var e = doc.createElement("menuitem");
     e.setAttribute("id", args["idVal"]);
-    var fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;
     e.setAttribute("label", args["labelId"]?fp.getMessage(args["labelId"], args["labelArgs"]) : args["labelVal"]);
     e.setAttribute("value", args["idVal"]);
     args["type"] && e.setAttribute("type", args["type"]);
@@ -190,7 +186,6 @@ Common.prototype = {
   },    
   
   onSuperAdd : function(wnd, url, superadd) {
-    var fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;
     var p = {inn:{url:url || this.getMostRecentWindow().content.location.href, superadd:superadd}, out:null};
     // superadd.proxy is null when user hasn't yet used QuickAdd
     if (superadd.proxy != null)
@@ -209,19 +204,30 @@ Common.prototype = {
       return p.match;
     }
   },
-  
-  makeProxyTreeView : function(fp, document) {    
+
+  makeProxyTreeView : function(proxies, document) {    
     var ret = {
-      rowCount : fp.proxies.length,
+      rowCount : proxies.length,
       getCellText : function(row, column) {
-        var i = fp.proxies.item(row);    
+        var i = proxies.item(row);    
         switch(column.id) {
           case "nameCol":return i.name;
           case "descriptionCol":return i.notes;
           case "hostCol":return i.manualconf.host;           
+          case "portCol":return i.manualconf.port; 
           case "isSocksCol":return i.manualconf.isSocks?fp.getMessage("yes"):fp.getMessage("no");        
-          case "portCol":return i.manualconf.port;                   
-          case "socksverCol":return i.manualconf.socksversion == "5" ? "5" : "4/4a";                           
+          case "socksverCol":
+            if (i.manualconf.isSocks)
+              return i.manualconf.socksversion == "5" ? "5" : "4/4a";
+            else
+              // We only want to show the SOCKS version in the tree if it is
+              // really a SOCKS proxy.
+              return "";
+          case "wpadCol":
+            if (i.mode === "wpad")
+              return fp.getMessage("yes");
+            else 
+              return fp.getMessage("no"); 
           case "autopacCol":return i.autoconf.url;   
           case "animatedIconsCol":return i.animatedIcons?fp.getMessage("yes"):fp.getMessage("no");
           case "cycleCol":return i.includeInCycle?fp.getMessage("yes"):fp.getMessage("no");
@@ -231,8 +237,8 @@ Common.prototype = {
               return i.proxyDNS ? fp.getMessage("yes"):fp.getMessage("no");
         }
       },
-      setCellValue: function(row, col, val) {fp.proxies.item(row).enabled = val;},
-      getCellValue: function(row, col) {return fp.proxies.item(row).enabled;},    
+      setCellValue: function(row, col, val) {proxies.item(row).enabled = val;},
+      getCellValue: function(row, col) {return proxies.item(row).enabled;},    
       isSeparator: function(aIndex) { return false; },
       isSorted: function() { return false; },
       isEditable: function(row, col) { return false; },
@@ -245,7 +251,7 @@ Common.prototype = {
       getColumnProperties: function(aColumn, aColumnElement, aProperty) {},
       getCellProperties: function(row, col, props) {
         if (col.id == "colorCol") {
-          var i = fp.proxies.item(row);
+          var i = proxies.item(row);
           var atom = CC["@mozilla.org/atom-service;1"].getService(CI.nsIAtomService).getAtom(i.colorString);
           props.AppendElement(atom);
         } 
@@ -258,40 +264,58 @@ Common.prototype = {
        so we can't use proxy.id or proxy.name or even proxy.color. Hence, the special
        proxy.colorString property that is a mapping of proxy.color to letters only
      */
-    var styleSheet = document.styleSheets[0], proxies = fp.proxies;
-    for (var i=0, len=fp.proxies.length; i<len; i++) {
-      var p = fp.proxies.item(i);
+    var styleSheet = document.styleSheets[0];
+    for (var i=0, len=proxies.length; i<len; i++) {
+      var p = proxies.item(i);
       styleSheet.insertRule("treechildren::-moz-tree-cell(" + p.colorString + "){border: 1px solid black;background-color:" + p.color + "}", styleSheet.cssRules.length);
     }
     return ret;
-  },
+  }, 
   
   isThunderbird : function() {
-    return CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULAppInfo).ID == "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
+    return this.appInfo.ID == "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
   },
   
-  notify : function(msg, buttons, callback) {
-    var wm = this.getMostRecentWindow(), nb = wm.gBrowser.getNotificationBox(),
-      n = nb.getNotificationWithValue("foxyproxy-proxy-scheme"),
-      fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject,
-      message = fp.getMessage(msg);
+  notify : function(msg, ar, buttons, priority, callback, getNotWithVal) {
+    let wm = this.getMostRecentWindow(), message = fp.getMessage(msg, ar), nb;
+    // First we check, whether we use Firefox or Seamonkey...
+    if (wm.gBrowser) {
+      nb = wm.gBrowser.getNotificationBox();
+    } else {
+      // We assume we are using Thunderbird now.
+      // TODO: We should optimize this a bit and should not use the main window
+      // notification if that method got called from proxy:// protocolhandler
+      // in Thunderbird.
+      nb = wm.document.getElementById("mail-notification-box");
+      // Should not happen but as a fallback we use a normal notification
+      // without buttons and are just showing the (error) message.
+      if (!nb) {
+        fp.notifier.alert(null, message);
+        return;
+      } 
+    }
+    if (getNotWithVal) {
+      getNotWithVal = nb.getNotificationWithValue("foxyproxy-notification");
+    }
     if (!buttons) {
       buttons = [
         { 
-          label: wm.gNavigatorBundle.getString("xpinstallPromptAllowButton"),
-          accessKey: wm.gNavigatorBundle.getString("xpinstallPromptAllowButton.accesskey"),
+          label: fp.getMessage("allow"),
+          accessKey: fp.getMessage("allow.accesskey"),
           popup: null, 
           callback: callback
         }                 
       ];  
     }
-    if (n) {
-      n.label = message;
+    if (!priority) {
+      priority = nb.PRIORITY_WARNING_MEDIUM;
+    } 
+    if (getNotWithVal) {
+      getNotWithVal.label = message;
     }
     else {
-      nb.appendNotification(message, "foxyproxy-proxy-scheme",
-          "chrome://foxyproxy/content/images/16x16.gif",
-          nb.PRIORITY_WARNING_MEDIUM, buttons);
+      nb.appendNotification(message, "foxyproxy-notification",
+          "chrome://foxyproxy/content/images/16x16.gif", priority, buttons);
     }
   },
   
@@ -317,4 +341,4 @@ Common.prototype = {
 if (XPCOMUtils.generateNSGetFactory)
   var NSGetFactory = XPCOMUtils.generateNSGetFactory([Common]);
 else
-  var NSGetModule = XPCOMUtils.generateNSGetModule([Common]);
\ No newline at end of file
+  var NSGetModule = XPCOMUtils.generateNSGetModule([Common]);
