@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 
 /* A better name for this class would have been Pattern */
@@ -30,9 +30,11 @@ if (!CI) {
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 ///////////////////////////// Match class///////////////////////
-function Match(enabled, name, pattern, temp, isRegEx, caseSensitive, isBlackList, isMultiLine) {
+function Match(enabled, name, pattern, temp, isRegEx, caseSensitive, isBlackList, isMultiLine, fromSubscription) {
   this.wrappedJSObject = this;
   this.init.apply(this, arguments);
+  Components.utils.import("resource://foxyproxy/patternSubscriptions.jsm", 
+    this); 
 }
 
 Match.prototype = {
@@ -44,10 +46,11 @@ Match.prototype = {
   _caseSensitive : false,
   isBlackList : false,
   _isMultiLine : false,
+  _fromSubscription : false,
 
   clone : function() {
     return new Match(this.enabled, this.name, this._pattern, this.temp, this.isRegEx, this.caseSensitive,
-      this.isBlackList, this._isMultiLine);
+      this.isBlackList, this._isMultiLine, this._fromSubscription);
   },
 
   init : function(enabled, name, pattern, temp, isRegEx, caseSensitive, isBlackList, isMultiLine) {
@@ -59,6 +62,7 @@ Match.prototype = {
     this._caseSensitive = arguments.length > 5 ? arguments[5] : false;
     this.isBlackList = arguments.length > 6 ? arguments[6] : false;
     this._isMultiLine = arguments.length > 7 ? arguments[7] : false;
+    this._fromSubscription = arguments.length > 8 ? arguments[8] : false;
     this.buildRegEx();
   },
 
@@ -99,27 +103,33 @@ Match.prototype = {
     return this._caseSensitive;
   },
 
+  set fromSubscription(m) {
+    this._fromSubscription = m;
+  },
+
+  get fromSubscription() {
+    return this._fromSubscription;
+  },
+
   buildRegEx : function() {
     var pat = this._pattern;
     if (!this._isRegEx) {
       // Wildcards
-      pat = pat.replace(/[$.+()^]/g, "\\$&"); /* $& replaces with the string found, but with that string escaped (like the .lastMatch property) */
+      // $& replaces with the string found, but with that string escaped (like
+      // the .lastMatch property)
+      pat = pat.replace(/[$.+()^]/g, "\\$&"); 
       pat = pat.replace(/\*/g, ".*");
       pat = pat.replace(/\?/g, ".");
     }
-    if (!this._isMultiLine) {
-	    pat[0] != "^" && (pat = "^" + pat);
-  	  pat[pat.length-1] != "$" && (pat = pat + "$");
-  	}
-  	try {
-	 	  this.regex = this._caseSensitive ? new RegExp(pat) : new RegExp(pat, "i");
-	 	}
-	 	catch(e){
-	 		// ignore--we might be in a state where the regexp is invalid because
-	 		// _isRegEx hasn't been changed to false yet, so we executed the wildcard
-	 		// replace() calls. however, this code is about to re-run because we'll be
-	 		// changed to a wildcard and re-calculate the regex correctly.
-	 	}
+    try {
+      this.regex = this._caseSensitive ? new RegExp(pat) : new RegExp(pat, "i");
+    }
+    catch(e) {
+      // ignore--we might be in a state where the regexp is invalid because
+      // _isRegEx hasn't been changed to false yet, so we executed the wildcard
+      // replace() calls. however, this code is about to re-run because we'll be
+      // changed to a wildcard and re-calculate the regex correctly.
+    }
   },
 
   fromDOM : function(n, includeTempPatterns) {
@@ -132,6 +142,7 @@ Match.prototype = {
 	  this._isMultiLine = gGetSafeAttrB(n, "isMultiLine", false);
     // Set this.caseSensitive instead of this._caseSensitive because the latter creates the regexp.
     this.caseSensitive = gGetSafeAttrB(n, "caseSensitive", false);
+    this._fromSubscription = gGetSafeAttrB(n, "fromSubscription", false);
     // We don't deserialize this.temp because it's not serialized
     // exception: use is copying these patterns (including temporary ones)
     // to a new proxy
@@ -149,10 +160,23 @@ Match.prototype = {
     matchElem.setAttribute("isBlackList", this.isBlackList);
     matchElem.setAttribute("isMultiLine", this._isMultiLine);
     matchElem.setAttribute("caseSensitive", this._caseSensitive);
+    matchElem.setAttribute("fromSubscription", this._fromSubscription);
     if (includeTempPatterns)
       matchElem.setAttribute("temp", this.temp);
     return matchElem;
   },
+
+  toJSON : function() {
+    let pattern = {};
+    pattern.enabled = this.enabled;
+    pattern.name = this.name;
+    pattern.pattern = this._pattern;
+    pattern.isRegEx = this.isRegEx;
+    pattern.caseSensitive = this._caseSensitive;
+    pattern.blackList = this.isBlackList;
+    pattern.multiLine = this._isMultiLine;
+    return this.patternSubscriptions.getJSONFromObject(pattern);
+  },
   
   QueryInterface: XPCOMUtils.generateQI([CI.nsISupports]),
   classDescription: "FoxyProxy Match Component",
