@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 //dump("proxy.js\n");
 if (!CI) {
@@ -74,11 +74,25 @@ function Proxy(fp) {
   this.name = this.notes = "";
   this.manualconf = new ManualConf(this, this.fp);
   this.autoconf = new AutoConf(this, this.fp);
+  // An own object for the WPAD feature...
+  this.wpad = new AutoConf(this, this.fp);
+  // We set a URL to the proxy file which cannot changed. The rationale for
+  // this is:
+  // "We diverge from the WPAD spec here in that we don't walk the
+  // hosts's FQDN, stripping components until we hit a TLD.  Doing so
+  // is dangerous in the face of an incomplete list of TLDs, and TLDs
+  // get added over time.  We could consider doing only a single
+  // substitution of the first component, if that proves to help
+  // compatibility." 
+  // See: http://mxr.mozilla.org/mozilla2.0/source/netwerk/base/src/
+  // nsProtocolProxyService.cpp#488 
+  this.wpad.url = "http://wpad/wpad.dat";
   this._mode = "manual"; // manual, auto, direct, random
   this._enabled = true;
   this.selectedTabIndex = 1; /* default tab is the proxy details tab */
   this.lastresort = false;
   this.id = this.fp.proxies.uniqueRandom();
+  this.noInternalIPs = false;
 }
 
 Proxy.prototype = {
@@ -98,6 +112,13 @@ Proxy.prototype = {
     this.notes = node.getAttribute("notes");
     this._enabled = node.getAttribute("enabled") == "true";
     this.autoconf.fromDOM(node.getElementsByTagName("autoconf").item(0));
+    let wpadNode = node.getElementsByTagName("autoconf").item(1);
+    if (wpadNode) { 
+      this.wpad.fromDOM(wpadNode); 
+    } else {
+      this.wpad = new AutoConf(this, this.fp);
+      this.wpad.url = "http://wpad/wpad.dat";
+    }
     this._proxyDNS = gGetSafeAttrB(node, "proxyDNS", true);
     this.manualconf.fromDOM(node.getElementsByTagName("manualconf").item(0));
     // 1.1 used "manual" instead of "mode" and was true/false only (for manual or auto)
@@ -113,6 +134,8 @@ Proxy.prototype = {
     this.includeInCycle = node.hasAttribute("includeInCycle") ? node.getAttribute("includeInCycle") == "true" : !this.lastresort; // new for 2.5
     this.color = gGetSafeAttr(node, "color", DEFAULT_COLOR);    
     
+    this.noInternalIPs = node.hasAttribute("noInternalIPs") ?
+      node.getAttribute("noInternalIPs") : false;
     for (var i=0,temp=node.getElementsByTagName("match"); i<temp.length; i++) {
       var j = this.matches.length;
       this.matches[j] = new Match();
@@ -141,6 +164,7 @@ Proxy.prototype = {
     e.setAttribute("includeInCycle", this.includeInCycle);
     e.setAttribute("color", this._color);
     e.setAttribute("proxyDNS", this._proxyDNS);
+    e.setAttribute("noInternalIPs", this.noInternalIPs);
 
     var matchesElem = doc.createElement("matches");
     e.appendChild(matchesElem);
@@ -148,6 +172,7 @@ Proxy.prototype = {
       if (!m.temp || (includeTempPatterns && m.temp)) matchesElem.appendChild(m.toDOM(doc, includeTempPatterns));
 
     e.appendChild(this.autoconf.toDOM(doc));
+    e.appendChild(this.wpad.toDOM(doc)); 
     e.appendChild(this.manualconf.toDOM(doc));
     return e;
   },
@@ -304,49 +329,79 @@ Proxy.prototype = {
   set enabled(e) {
     if (this.lastresort && !e) return; // can't ever disable this guy
     this._enabled = e;
-		this.shouldLoadPAC() && this.autoconf.loadPAC();
+    if (this.shouldLoadPAC()) {
+      if (this._mode === "auto") {
+        this.autoconf.loadPAC();
+      } else {
+        this.wpad.loadPAC();
+      }
+    } 
     this.handleTimer();
   },
 
   get enabled() {return this._enabled;},
 
-	shouldLoadPAC : function() {
-    if (this._mode == "auto" && this._enabled) {
+  shouldLoadPAC : function() {
+    if ((this._mode == "auto" || this._mode == "wpad") &&
+         this._enabled) {
       var m = this.fp.mode;
-      return m == this.id || m == "patterns" || m == "random" || m == "roundrobin";
+      return m == this.id || m == "patterns" || m == "random" ||
+        m == "roundrobin";
     }
-	},
+  },
 
   set mode(m) {
     this._mode = m;
-    this.shouldLoadPAC() && this.autoconf.loadPAC();
+    if (this.shouldLoadPAC()) {
+      if (this._mode === "auto") {
+        this.autoconf.loadPAC();
+      } else {
+        this.wpad.loadPAC();
+      }
+    } 
     this.handleTimer();
   },
 
-	afterPropertiesSet : function() {
-	  // Load PAC if required. Note that loadPAC() is synchronous and if it fails, it changes our mode to "direct" or disables us.
-    this.shouldLoadPAC() && this.autoconf.loadPAC();
-
-   	// Some integrity maintenance: if this is a manual proxy and this.manualconf.proxy wasn't created during deserialization, disable us.
+  afterPropertiesSet : function() {
+    // Load PAC if required. Note that loadPAC() is synchronous and if it fails,
+    // it changes our mode to "direct" or disables us.
+    if (this.shouldLoadPAC()) {
+      if (this._mode === "auto") {
+        this.autoconf.loadPAC();
+      } else {
+        this.wpad.loadPAC();
+      }
+    } 
+    // Some integrity maintenance: if this is a manual proxy and
+    // this.manualconf.proxy wasn't created during deserialization, disable us.
     if (this._enabled && this._mode == "manual" && !this.manualconf.proxy) {
       if (this.lastresort) {
-     	  // Switch lastresort to DIRECT since manualconf is corrupt--someone changed foxyproxy.xml manually, outside our GUI
-     	  this._mode = "direct";
+        // Switch lastresort to DIRECT since manualconf is corrupt--someone
+        // changed foxyproxy.xml manually, outside our GUI
+        this._mode = "direct";
+      } else {
+        this._enabled = false;
       }
-      else
-     	  this._enabled = false;
+      !this._enabled &&
+        // (proxy, isBeingDeleted, isBeingDisabled, isBecomingDIRECT)  
+        this.fp.proxies.maintainIntegrity(this, false, true, false); 
     }
-	  !this._enabled &&
-	 	  this.fp.proxies.maintainIntegrity(this, false, true, false); // (proxy, isBeingDeleted, isBeingDisabled, isBecomingDIRECT)
-	},
+  },
 
-	handleTimer : function() {
-		var ac = this.autoconf;
-		ac.timer.cancel(); // always always always cancel first before doing anything
-		if (this.shouldLoadPAC() && ac._autoReload) {
-			ac.timer.initWithCallback(ac, ac._reloadFreqMins*60000, CI.nsITimer.TYPE_REPEATING_SLACK);
-		}
-	},
+  handleTimer : function() {
+    let ac;
+    if (this._mode === "auto") {
+      ac = this.autoconf; 
+    } else {
+      ac = this.wpad;
+    } 
+    // always always always cancel first before doing anything 
+    ac.timer.cancel();
+    if (this.shouldLoadPAC() && ac._autoReload) {
+      ac.timer.initWithCallback(ac, ac._reloadFreqMins*60000,
+        CI.nsITimer.TYPE_REPEATING_SLACK);
+    }
+  },
 
   get mode() {return this._mode;},
 
@@ -390,66 +445,82 @@ Proxy.prototype = {
     this.matches = this.matches.filter(function(e) {return e != removeMe;});
   },
   
-	resolve : function(spec, host, mp) {
-	  function _notifyUserOfError(spec) {
-			/*this.autoconf.errorNotification &&*/ this.fp.notifier.alert(this.fp.getMessage("foxyproxy"), this.fp.getMessage("proxy.error.for.url", [spec]));
-			return null;
-		}
-	  // See http://wp.netscape.com/eng/mozilla/2.0/relnotes/demo/proxy-live.html
-	  var str = mp.pacResult = this.autoconf._resolver.getProxyForURI(spec, host);
-	  if (str && str != "") {
-	    str = str.toLowerCase();
-	    var tokens = str.split(/\s*;\s*/), // Trim and split
-      	proxies = [];
-	    if (tokens[tokens.length-1] == "") // In case final token ends with semi-colon
-	      tokens.length--;
-	    for (var i=0; i<tokens.length; i++) {
-	      var components = this.autoconf.parser.exec(tokens[i]);
-	      if (!components) continue;
-	      var tmp = this._proxyDNS ? CI.nsIProxyInfo.TRANSPARENT_PROXY_RESOLVES_HOST : 0;
-	      switch (components[1]) {
-	        case "proxy":
-	          proxies.push(proxyService.newProxyInfo("http", components[2], components[3], tmp, 0, null));
-	          break;
-	        case "socks":
-	        case "socks5":
-	          proxies.push(proxyService.newProxyInfo("socks", components[2], components[3], tmp, 0, null));
-	          break;
-	        case "socks4":
-	          proxies.push(proxyService.newProxyInfo("socks4", components[2], components[3], tmp, 0, null));
-	          break;
-	        case "direct":
-	          proxies.push(this.direct);
-	          break;
-	        default:
-	          return _notifyUserOfError(spec);
-	      }
-	    }
-	    // Build a proxy list for proxy for failover support
-	    for (var i=1; i<=proxies.length-1; i++) {
-	      proxies[i-1].failoverTimeout = 1800;
-	      proxies[i-1].failoverProxy = proxies[i];
-	    }
-	    if (proxies[0] == null) {
-		    return _notifyUserOfError(spec);
-		  }
-		  else if (proxies[1]) {
-			  proxies[0].failoverTimeout = 1800;
-			  proxies[0].failoverProxy = proxies[1];
-		  }
-	    return proxies[0];
-	  }
-	  else {
-	    // Resolver did not find a proxy, but this isn't an error condition
-	    return null;
-	  }
-	},
+  resolve : function(spec, host, mp, isWPAD) {
+    function _notifyUserOfError(spec) {
+      /*this.autoconf.errorNotification &&*/
+      this.fp.notifier.alert(this.fp.getMessage("foxyproxy"),
+        this.fp.getMessage("proxy.error.for.url", [spec]));
+      return null;
+    }
+    // See http://wp.netscape.com/eng/mozilla/2.0/relnotes/demo/proxy-live.html
+    if (isWPAD) {
+      var str = mp.pacResult = this.wpad._resolver.getProxyForURI(spec, host);
+    } else {
+      var str = mp.pacResult = this.autoconf._resolver.getProxyForURI(spec,
+        host);
+    }
+    if (str && str != "") {
+      str = str.toLowerCase();
+      var tokens = str.split(/\s*;\s*/), // Trim and split
+      proxies = [];
+      // In case final token ends with semi-colon 
+      if (tokens[tokens.length-1] == "")
+        tokens.length--;
+      for (var i=0; i<tokens.length; i++) {
+        if (isWPAD) {
+          var components = this.wpad.parser.exec(tokens[i]);
+        } else {
+          var components = this.autoconf.parser.exec(tokens[i]); 
+        }
+        if (!components) continue;
+        var tmp = this._proxyDNS && components[1].indexOf("socks") === 0 ?
+          CI.nsIProxyInfo.TRANSPARENT_PROXY_RESOLVES_HOST : 0;
+        switch (components[1]) {
+          case "proxy":
+            proxies.push(proxyService.newProxyInfo("http", components[2],
+              components[3], tmp, 0, null));
+            break;
+          case "socks":
+          case "socks5":
+            proxies.push(proxyService.newProxyInfo("socks", components[2],
+              components[3], tmp, 0, null));
+            break;
+          case "socks4":
+            proxies.push(proxyService.newProxyInfo("socks4", components[2],
+              components[3], tmp, 0, null));
+            break;
+          case "direct":
+            proxies.push(this.direct);
+           break;
+          default:
+            return _notifyUserOfError(spec);
+        }
+      }
+      // Build a proxy list for proxy for failover support
+      for (var i=1; i<=proxies.length-1; i++) {
+        proxies[i-1].failoverTimeout = 1800;
+        proxies[i-1].failoverProxy = proxies[i];
+      }
+      if (proxies[0] == null) {
+        return _notifyUserOfError(spec);
+      }
+      else if (proxies[1]) {
+        proxies[0].failoverTimeout = 1800;
+        proxies[0].failoverProxy = proxies[1];
+      }
+      return proxies[0];
+    } else {
+      // Resolver did not find a proxy, but this isn't an error condition
+      return null;
+    }
+  },
 
   getProxy : function(spec, host, mp) {
     switch (this._mode) {
       case "manual":return this.manualconf.proxy;
-      case "auto":return this.resolve(spec, host, mp);
-	    case "direct":return this.direct;
+      case "wpad":return this.resolve(spec, host, mp, true);
+      case "auto":return this.resolve(spec, host, mp, false);
+      case "direct":return this.direct;
     }
   },
   
