@@ -6,10 +6,11 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 const CI = Components.interfaces, CC = Components.classes;
-var urlsTree, proxy, foxyproxy, autoconfurl, overlay, isWindows, fpc;
+var urlsTree, proxy, foxyproxy, autoconfurl, overlay, isWindows, fpc,
+  oldMatches = [];
 
 function onLoad() {
   isWindows = CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULRuntime).OS == "WINNT";
@@ -17,15 +18,17 @@ function onLoad() {
   overlay = fpc.getMostRecentWindow().foxyproxy;
   autoconfurl = document.getElementById("autoconfurl");
   foxyproxy = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;
+  proxy = window.arguments[0].inn.proxy; 
   if (window.arguments[0].inn.torwiz) {
     document.getElementById("torwiz-broadcaster").hidden = true;
     document.getElementById("not-torwiz-broadcaster").hidden = false;
     urlsTree = document.getElementById("torWizUrlsTree");
   }
-  else
+  else {
     urlsTree = document.getElementById("urlsTree");
+    document.getElementById("noInternalIPs").checked = proxy.noInternalIPs;
+  }
 
-  proxy = window.arguments[0].inn.proxy;
   document.getElementById("proxyname").value = proxy.name;
   document.getElementById("proxynotes").value = proxy.notes;
   document.getElementById("animatedIcons").checked = proxy.animatedIcons;
@@ -35,12 +38,14 @@ function onLoad() {
   document.getElementById("tabs").selectedIndex = proxy.selectedTabIndex;
   document.getElementById("proxyenabled").checked = proxy.enabled;
   document.getElementById("mode").value = proxy.mode;
-  toggleMode(proxy.mode);
   document.getElementById("host").value = proxy.manualconf.host;
   document.getElementById("port").value = proxy.manualconf.port;
   document.getElementById("isSocks").checked = proxy.manualconf.isSocks;
   document.getElementById("socksversion").value = proxy.manualconf.socksversion;
+  toggleMode(proxy.mode); 
   document.getElementById("proxyDNS").checked = proxy.proxyDNS;
+  document.getElementById("WPADReloadEnabled").checked = proxy.wpad.autoReload;
+  document.getElementById("WPADReloadFreq").value = proxy.wpad.reloadFreqMins;
   autoconfurl.value = proxy.autoconf.url;
 
   if (proxy.lastresort) {
@@ -53,11 +58,27 @@ function onLoad() {
   document.getElementById("pacErrorNotificationEnabled").checked = proxy.autoconf.errorNotification;
   document.getElementById("autoConfURLReloadEnabled").checked = proxy.autoconf.autoReload;
   document.getElementById("autoConfReloadFreq").value = proxy.autoconf.reloadFreqMins;
-
+  // We need to copy the matches array here in order to replace the modified
+  // one with the old one if the user does not press the OK button (e.g. using
+  // the Cancel button or just closing the window manually).
+  for (let i = 0, length = proxy.matches.length; i < length; i++) {
+    oldMatches[i] = proxy.matches[i].clone(); 
+  }
   _updateView();
   sizeToContent();
 }
 
+function onCancel() {
+  // We just overwrite the new array with the old one. Checking whether any
+  // pattern really changed before overwriting the new array seems not worth
+  // the effort. 
+  proxy.matches = [];
+  for (let i = 0, length = oldMatches.length; i < length; i++) {
+    proxy.matches.push(oldMatches[i]);
+  } 
+  return true;
+}
+
 function trim(s) {
 	return s.replace(/^\s*|\s*$/g, "");
 }
@@ -108,11 +129,13 @@ function onOK() {
   proxy.name = name;
   proxy.notes = document.getElementById("proxynotes").value;
   proxy.selectedTabIndex = document.getElementById("tabs").selectedIndex;
+  proxy.wpad.autoReload = document.getElementById("WPADReloadEnabled").checked;
+  proxy.wpad.reloadFreqMins = document.getElementById("WPADReloadFreq").value;
   proxy.autoconf.url = url;
   proxy.autoconf.loadNotification = document.getElementById("pacLoadNotificationEnabled").checked;
   proxy.autoconf.errorNotification = document.getElementById("pacErrorNotificationEnabled").checked;
-	proxy.autoconf.autoReload = document.getElementById("autoConfURLReloadEnabled").checked;
-	proxy.autoconf.reloadFreqMins = reloadfreq;
+  proxy.autoconf.autoReload = document.getElementById("autoConfURLReloadEnabled").checked;
+  proxy.autoconf.reloadFreqMins = reloadfreq;
 
   proxy.mode = mode; // set this first to control PAC loading
   proxy.enabled = enabled;
@@ -130,6 +153,11 @@ function onOK() {
     return false;
   }
   proxy.proxyDNS = document.getElementById("proxyDNS").checked;
+  if (window.arguments[0].inn.torwiz) {
+    proxy.noInternalIPs = document.getElementById("fpniip").checked;
+  } else {
+    proxy.noInternalIPs = document.getElementById("noInternalIPs").checked;
+  }
   proxy.afterPropertiesSet();
   window.arguments[0].out = {proxy:proxy};
   return true;
@@ -159,6 +187,43 @@ function _checkUri() {
   }
 }
 
+function noInternalIPs() {
+  let noInternalIPsChecked;
+  if (window.arguments[0].inn.torwiz) {
+    noInternalIPsChecked = document.getElementById("fpniip").checked;
+  } else {
+    noInternalIPsChecked = document.getElementById("noInternalIPs").checked;
+  }
+  if (noInternalIPsChecked) {
+    let helper = [];
+    let m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
+      wrappedJSObject; 
+    m.init(true, foxyproxy.getMessage("localhost2") +
+      foxyproxy.getMessage("localhost.patterns.message"),
+      "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:localhost|127\.\d+\.\d+\.\d+)(?::\d+)?/.*",
+      false, true, false, true, false);
+    helper.push(m);
+    m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
+      wrappedJSObject; 
+    m.init(true, foxyproxy.getMessage("localsubnets2") +
+      foxyproxy.getMessage("localhost.patterns.message"), "^https?://(?:[^:@/]+(?::[^@/]+)?@)?(?:192\.168\.\d+\.\d+|10\.\d+\.\d+\.\d+|172\.(?:1[6789]|2[0-9]|3[12]))(?::\d+)?/.*",
+      false, true, false, true, false);
+    helper.push(m);
+    m = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
+      wrappedJSObject; 
+    m.init(true, foxyproxy.getMessage("localhostnames2") +
+      foxyproxy.getMessage("localhost.patterns.message"),
+      "^https?://(?:[^:@/]+(?::[^@/]+)?@)?[\w-]+(?::\d+)?/.*", false, true,
+      false, true, false);
+    helper.push(m);
+    proxy.matches = helper.concat(proxy.matches); 
+    _updateView();
+  } else {
+    proxy.matches = proxy.matches.slice(3);
+    _updateView();
+  }
+}
+
 function onAddEditURLPattern(isNew) {
   var idx = urlsTree.currentIndex, m;
   if (isNew) {
@@ -193,10 +258,16 @@ function getTextForCell(pat, col) {
     case "isBlackList":return foxyproxy.getMessage(pat.isBlackList ? "foxyproxy.blacklist.label" : "foxyproxy.whitelist.label");
     case "caseSensitive":return foxyproxy.getMessage(pat.caseSensitive ? "yes" : "no");
     case "temp":return foxyproxy.getMessage(pat.temp ? "yes" : "no");
+    case "fromSubscription":return foxyproxy.
+      getMessage(pat.fromSubscription ? "yes" : "no");
   };
 }
 
 function _updateView() {
+  // We disable and enable the JSON export button depending on available
+  // patterns.
+  document.getElementById("exportURLPatternCmd").setAttribute("disabled", 
+    proxy.matches.length === 0); 
   // Redraw the trees
   urlsTree.view = makeView();
 
@@ -251,27 +322,149 @@ function onCopyURLPattern() {
   urlsTree.view.selection.select(urlsTree.view.rowCount-1);
 }
 
+function onImportURLPattern() {
+  //Getting the file first.
+  let fp = CC["@mozilla.org/filepicker;1"].createInstance(CI.nsIFilePicker);  
+  fp.init(window, foxyproxy.getMessage("file.select.patterns.import"), 
+    CI.nsIFilePicker.modeOpen);
+  fp.appendFilters(CI.nsIFilePicker.filterAll); 
+  fp.displayDirectory = foxyproxy.getSettingsURI(CI.nsIFile).parent; 
+  if (fp.show() !== CI.nsIFilePicker.returnCancel) { 
+    let patterns = [];
+    let fis = CC["@mozilla.org/network/file-input-stream;1"].
+      createInstance(CI.nsIFileInputStream);
+    fis.init(fp.file, 0x01, -1, 0);
+    let conStream = CC["@mozilla.org/intl/converter-input-stream;1"].
+      createInstance(CI.nsIConverterInputStream);
+      conStream.init(fis, "UTF-8", 0, 0);
+      conStream.QueryInterface(CI.nsIUnicharLineInputStream); 
+    // read lines into one single string
+    let line = {}, lines = "", hasmore;
+    try {
+      do {
+        hasmore = conStream.readLine(line);
+        lines = lines + line.value;
+      } while(hasmore);
+      conStream.close();
+      patterns = window.opener.patternSubscriptions.getObjectFromJSON(lines).
+        patterns;
+    } catch (e) {
+      dump("Error while reading the patterns!" + e + "\n");
+    } 
+    try {
+      if (patterns) {
+        for (let i = 0, patLength = patterns.length; i < patLength; i++) {
+          pattern = CC["@leahscape.org/foxyproxy/match;1"].createInstance().
+                    wrappedJSObject; 
+          pattern.init(patterns[i].enabled, patterns[i].name, 
+                    patterns[i].pattern, false, patterns[i].isRegEx, 
+                    patterns[i].caseSensitive, patterns[i].blackList, 
+                    patterns[i].multiLine, false);
+          proxy.matches.push(pattern);
+        }
+      }
+    } catch (ex) {
+      dump("Error while adding the patterns!" + ex + "\n");
+    }
+  }
+  _updateView();
+}
+
+function onExportURLPattern() {
+  let patternLength = proxy.matches.length;
+  // Now we are constructing the pattern JSON
+  let JSONString = '{"patterns":[';
+  for (let j = 0; j < patternLength; j++) {
+    JSONString = JSONString + proxy.matches[j].toJSON();
+    if (j < patternLength - 1) {
+      JSONString = JSONString + ",";
+    } else {
+      JSONString = JSONString + "]}";
+    }
+  }
+  // Now, we export the JSON to a file somewhere on the (local) disk...
+  let fp = CC["@mozilla.org/filepicker;1"].createInstance(CI.nsIFilePicker);  
+  fp.init(window, foxyproxy.getMessage("file.select.patterns.export"), 
+    CI.nsIFilePicker.modeSave);
+  fp.defaultString = "patterns.json";
+  fp.appendFilters(CI.nsIFilePicker.filterAll); 
+  fp.displayDirectory = foxyproxy.getSettingsURI(CI.nsIFile).parent; 
+  if (fp.show() !== CI.nsIFilePicker.returnCancel) { 
+    let fos = CC["@mozilla.org/network/file-output-stream;1"].
+      createInstance(CI.nsIFileOutputStream); 
+    fos.init(fp.file, 0x02 | 0x08 | 0x20, -1, 0); 
+    // Maybe we have non-Ascii text in our JSON string. Therefore, we use the
+    // ConverterOutputStream and UTF-8.
+    let os = CC["@mozilla.org/intl/converter-output-stream;1"].
+      createInstance(CI.nsIConverterOutputStream);	
+    os.init(fos, "UTF-8", 0, 0);
+    os.writeString(JSONString);
+    os.close();
+  }
+}
+
+function toggleSocks() {
+  let socksBC = document.getElementById("socks-broadcaster");
+  if (document.getElementById("isSocks").checked) {
+    socksBC.removeAttribute("disabled"); 
+  } else {
+    socksBC.setAttribute("disabled", "true");
+  }
+}
+
 function toggleMode(mode) {
   // Next line--buggy in FF 1.5.0.1--makes fields enabled but readonly
   // document.getElementById("disabled-broadcaster").setAttribute("disabled", mode == "auto" ? "true" : "false");
   // Call removeAttribute() instead of setAttribute("disabled", "false") or setAttribute("disabled", false);
   // Thanks, Andy McDonald.
   if (mode == "auto") {
-    document.getElementById("autoconf-broadcaster1").removeAttribute("disabled");
-		document.getElementById("disabled-broadcaster").setAttribute("disabled", "true");
-		document.getElementById("direct-broadcaster").removeAttribute("disabled", "true");
-		document.getElementById("proxyDNS").hidden = false;
-		onAutoConfUrlInput();
-  }
-  else if (mode == "direct") {
-    document.getElementById("disabled-broadcaster").setAttribute("disabled", "true");
-		document.getElementById("autoconf-broadcaster1").setAttribute("disabled", "true");
-		document.getElementById("direct-broadcaster").setAttribute("disabled", "true");
-		document.getElementById("proxyDNS").hidden = true;
-  }
-  else {
+    document.getElementById("autoconf-broadcaster1").
+      removeAttribute("disabled");
+    document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
+      "true");
+    document.getElementById("socks-broadcaster").setAttribute("disabled",
+      "true");
+    document.getElementById("disabled-broadcaster").setAttribute("disabled",
+      "true");
+    document.getElementById("direct-broadcaster").removeAttribute("disabled");
+    document.getElementById("proxyDNS").hidden = false;
+    onAutoConfUrlInput();
+  } else if (mode == "direct") {
+    document.getElementById("disabled-broadcaster").setAttribute("disabled",
+      "true");
+    document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
+      "true");
+    document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
+      "true"); 
+    document.getElementById("socks-broadcaster").setAttribute("disabled",
+      "true"); 
+    document.getElementById("direct-broadcaster").setAttribute("disabled",
+      "true");
+    document.getElementById("proxyDNS").hidden = true;
+  } else if (mode == "wpad") {
+    document.getElementById("disabled-broadcaster").setAttribute("disabled",
+      "true");
+    document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
+      "true");
+    document.getElementById("autoconf-broadcaster3").
+      removeAttribute("disabled");
+    document.getElementById("socks-broadcaster").setAttribute("disabled",
+      "true"); 
+    document.getElementById("direct-broadcaster").removeAttribute("disabled")
+    document.getElementById("proxyDNS").hidden = false; 
+  
+  } else {
     document.getElementById("disabled-broadcaster").removeAttribute("disabled");
-    document.getElementById("autoconf-broadcaster1").setAttribute("disabled", "true");
+    document.getElementById("autoconf-broadcaster1").setAttribute("disabled",
+      "true");
+    document.getElementById("autoconf-broadcaster3").setAttribute("disabled",
+      "true"); 
+    if (document.getElementById("isSocks").checked) {
+      document.getElementById("socks-broadcaster").removeAttribute("disabled");
+    } else {
+      document.getElementById("socks-broadcaster").setAttribute("disabled",
+        "true"); 
+    }
     document.getElementById("direct-broadcaster").removeAttribute("disabled");
     document.getElementById("proxyDNS").hidden = false;
   }
@@ -439,4 +632,4 @@ function onHostChange(hostInput) {
       portInput.value = port;
     }
   }
-}
\ No newline at end of file
+}
