@@ -6,33 +6,49 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
-var proxyTree, fp, fpc, overlay, inn;
+var proxyTree, fp, fpc, inn;
 const CC = Components.classes, CI = Components.interfaces;
 
 function onLoad() {
   proxyTree = document.getElementById("proxyTree");
   fp = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;  
   fpc = CC["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject;
-  overlay = fpc.getMostRecentWindow().foxyproxy;
   inn = window.arguments[0].inn;
-  
+
+  // Using the chooseproxy.xul for pattern subscription purposes we do
+  // neither need the extra2 button nor the reloadcurtab-checkbox. Thus,
+  // collapsing them.
+  if (inn.pattern) {
+    document.documentElement.getButton("extra2").collapsed = true;
+    document.getElementById("reloadcurtab").collapsed = true;
+  }
+
   // Append title as a textnode to the description so text wrapping works
   var title = document.getElementById("title");
   title.appendChild(document.createTextNode(inn.title));
   
-  proxyTree.view = fpc.makeProxyTreeView(fp, document);
+  proxyTree.view = fpc.makeProxyTreeView(fp.proxies, document);
   proxyTree.view.selection.select(0); /* select the first entry */
-  document.getElementById("reloadcurtab").checked = inn.reloadcurtab;
+  if (!inn.pattern) {
+    document.getElementById("reloadcurtab").checked = inn.reloadcurtab;
+  }
   sizeToContent();
 }
 
 function onOK() {
-  if (proxyTree.currentIndex != -1)
-    window.arguments[0].out = {proxy:fp.proxies.item(proxyTree.currentIndex), reloadcurtab:
+  if (proxyTree.currentIndex !== -1) {
+    if (inn.pattern && fp.proxies.item(proxyTree.currentIndex).lastresort) { 
+      fp.alert(window, fp.getMessage("patternsubscription.pattern.default")); 
+      return false;
+    }
+    window.arguments[0].out = {proxy:fp.proxies.item(proxyTree.currentIndex), 
+      reloadcurtab:
       document.getElementById("reloadcurtab").checked};
-  return true;
+    return true;
+  }
+  return false;
 }
 
 function onSettings() {
@@ -46,11 +62,11 @@ function onSettings() {
     "chrome, dialog, modal, resizable=yes", params).focus();
   if (params.out) {
     fp.proxies.push(params.out.proxy);
-    proxyTree.view = fpc.makeProxyTreeView(fp, document); /* reset the view to show the new entry */
+    proxyTree.view = fpc.makeProxyTreeView(fp.proxies, document); /* reset the view to show the new entry */
     fp.writeSettings();
   }
 
   // Reselect what was previously selected or the new item
   proxyTree.view.selection.select(proxyTree.view.rowCount-2);
   //document.documentElement.acceptDialog();
-}
\ No newline at end of file
+}
