@@ -6,43 +6,42 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 
-var foxyproxy, proxyTree, logTree, monthslong, dayslong, overlay, timeformat, saveLogCmd, clearLogCmd, noURLsCmd, fpc;
-const CI = Components.interfaces, CC = Components.classes;
+var foxyproxy, proxyTree, subscriptionsTree, logTree, overlay, saveLogCmd, 
+  clearLogCmd, noURLsCmd, fpc, patternsIcon; 
+const CI = Components.interfaces, CC = Components.classes, CU = Components.utils;
+
+CU.import("resource://foxyproxy/patternSubscriptions.jsm");
 
 function onLoad() {
   foxyproxy = CC["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject;  
   fpc = CC["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject;
   document.getElementById("maxSize").value = foxyproxy.logg.maxSize;
   overlay = fpc.getMostRecentWindow().foxyproxy;
-  monthslong = [foxyproxy.getMessage("months.long.1"), foxyproxy.getMessage("months.long.2"),
-    foxyproxy.getMessage("months.long.3"), foxyproxy.getMessage("months.long.4"), foxyproxy.getMessage("months.long.5"),
-    foxyproxy.getMessage("months.long.6"), foxyproxy.getMessage("months.long.7"), foxyproxy.getMessage("months.long.8"),
-    foxyproxy.getMessage("months.long.9"), foxyproxy.getMessage("months.long.10"), foxyproxy.getMessage("months.long.11"),
-    foxyproxy.getMessage("months.long.12")];
-  
-  dayslong = [foxyproxy.getMessage("days.long.1"), foxyproxy.getMessage("days.long.2"),
-    foxyproxy.getMessage("days.long.3"), foxyproxy.getMessage("days.long.4"), foxyproxy.getMessage("days.long.5"),
-    foxyproxy.getMessage("days.long.6"), foxyproxy.getMessage("days.long.7")];
   proxyTree = document.getElementById("proxyTree");
+  subscriptionsTree = document.getElementById("subscriptionsTree");
   logTree = document.getElementById("logTree");
   saveLogCmd = document.getElementById("saveLogCmd");
   clearLogCmd = document.getElementById("clearLogCmd");  
   noURLsCmd = document.getElementById("noURLsCmd");
-  timeformat = foxyproxy.getMessage("timeformat");
   _initSettings();
   var obs = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
   obs.addObserver(observer,"foxyproxy-mode-change", false);
   obs.addObserver(observer,"foxyproxy-proxy-change", false);
+  obs.addObserver(observer,"foxyproxy-tree-update", false);
   sizeToContent();
+  let svgDoc = document.getElementById("patterns-svg");
+  patternsIcon = svgDoc.getElementsByTagName("rect");
+  createPatternsIcon();
 }
 
 function onOK() {
   var obs = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
   obs.removeObserver(observer, "foxyproxy-mode-change");
   obs.removeObserver(observer, "foxyproxy-proxy-change");
+  obs.removeObserver(observer, "foxyproxy-tree-update");
 }
 
 var observer = {
@@ -57,6 +56,10 @@ var observer = {
       case "foxyproxy-mode-change":
         _updateView(e);
         break;
+      case "foxyproxy-tree-update":
+        subscriptionsTree.view = patternSubscriptions.
+          makeSubscriptionsTreeView();
+        break;
      }
    }
 }
@@ -135,8 +138,8 @@ function _updateLogView(keepSelection) {
     getCellText : function(row, column) {
       var mp = foxyproxy.logg.item(row);
       if (!mp) return;
-      if (column.id == "timestamp") return format(mp.timestamp);
-      return mp[column.id];
+      if (column.id == "timestamp") return foxyproxy.logg.format(mp.timestamp);
+      return mp[column.id] ? mp[column.id] : null;
     },
     isSeparator: function(aIndex) { return false; },
     isSorted: function() { return false; },
@@ -173,36 +176,6 @@ function _updateLogView(keepSelection) {
   updateLogButtons();
 }
 
-  // Thanks for the inspiration, Tor2k (http://www.codeproject.com/jscript/dateformat.asp)
-  function format(d) {
-    d = new Date(d);
-    if (!d.valueOf())
-      return '&nbsp;';
-
-    return timeformat.replace(/(yyyy|mmmm|mmm|mm|dddd|ddd|dd|hh|HH|nn|ss|zzz|a\/p)/gi,
-      function($1) {
-        switch ($1) {
-          case 'yyyy': return d.getFullYear();
-          case 'mmmm': return monthslong[d.getMonth()];
-          case 'mmm':  return monthslong[d.getMonth()].substr(0, 3);
-          case 'mm':   return zf((d.getMonth() + 1), 2);
-          case 'dddd': return dayslong[d.getDay()];
-          case 'ddd':  return dayslong[d.getDay()].substr(0, 3);
-          case 'dd':   return zf(d.getDate(), 2);
-          case 'hh':   return zf(((h = d.getHours() % 12) ? h : 12), 2);
-          case 'HH':   return zf(d.getHours(), 2);          
-          case 'nn':   return zf(d.getMinutes(), 2);
-          case 'ss':   return zf(d.getSeconds(), 2);
-          case 'zzz':  return zf(d.getSeconds(), 3);          
-          case 'a/p':  return d.getHours() < 12 ? 'AM' : 'PM';
-        }
-      }
-    );
-  }
-  
-// My own zero-fill fcn, not Tor 2k's. Assumes (n==2 || n == 3) && c<=n.
-function zf(c, n) { c=""+c; return c.length == 1 ? (n==2?'0'+c:'00'+c) : (c.length == 2 ? (n==2?c:'0'+c) : c); }
-
 function _updateModeMenu() {
 	var menu = document.getElementById("modeMenu");	
 	var popup=menu.firstChild;
@@ -212,7 +185,7 @@ function _updateModeMenu() {
 	  popup.appendChild(fpc.createMenuItem({idVal:"patterns", labelId:"mode.patterns.label", document:document}));
   
   for (var i=0,p; i<foxyproxy.proxies.length && ((p=foxyproxy.proxies.item(i)) || 1); i++)
-    popup.appendChild(fpc.createMenuItem({idVal:p.id, labelId:"mode.custom.label", labelArgs:[p.name], type:"radio", name:"foxyproxy-enabled-type", document:document}));
+    popup.appendChild(fpc.createMenuItem({idVal:p.id, labelId:"mode.custom.label", labelArgs:[p.name], name:"foxyproxy-enabled-type", document:document}));
     //popup.appendChild(fpc.createMenuItem({idVal["random", labelId:"mode.random.label", document:document}));
   popup.appendChild(fpc.createMenuItem({idVal:"disabled", labelId:"mode.disabled.label", document:document}));
   menu.value = foxyproxy.mode;
@@ -234,14 +207,14 @@ function onSettingsURLBtn() {
   fp.init(window, foxyproxy.getMessage("file.select"), nsIFilePicker.modeSave);
   fp.defaultString = "foxyproxy.xml";
   fp.appendFilters(nsIFilePicker.filterAll|nsIFilePicker.filterXML);
-  fp.displayDirectory = foxyproxy.getSettingsURI(CI.nsIFile); /* WHY IS THIS ALWAYS NULL? */
+  // nsIFilePicker.displayDirectory requires a directory WITHOUT a filename
+  // appended. Hence we use .parent here and in the following lines of code.
+  fp.displayDirectory = foxyproxy.getSettingsURI(CI.nsIFile).parent; 
   if (fp.show() != nsIFilePicker.returnCancel) {
     var defPath = foxyproxy.getDefaultPath();
     // If the current settings file is in the default path and the user wants to move it, warn him.
-    // Since foxyproxy.getSettingsURI(CI.nsIFile) is always evaluating to null is this context (WHY?!),
-    // I'm skipping the first conditional expression.
-    //if (fp.displayDirectory.equals(defPath) && !defPath.equals(fp.file)) {
-    if (!defPath.equals(fp.file)) {
+    if (fp.displayDirectory.equals(defPath.parent) && !defPath.parent.
+        equals(fp.file.parent)) {
       var c = overlay.ask(this, foxyproxy.getMessage("settings.warning"), null, null, foxyproxy.getMessage("more.info"));
       switch (c) {
         case 1:
@@ -254,14 +227,26 @@ function onSettingsURLBtn() {
       }
     }
     foxyproxy.setSettingsURI(fp.file);
+    // We write the pattern subscriptions as well to the new place (if there 
+    // are any). We do not do this in the general writeSubscriptions method in
+    // foxyproxy.js as this would only imply unnecessary disk I/O (the 
+    // pattern subscriptions would then always be written, even if any tiny 
+    // and unrelated setting was changed).
+    if (patternSubscriptions.subscriptionsList.length > 0) {
+     patternSubscriptions.writeSubscriptions();
+    }
     _initSettings();
   }
 }
 
 function onResetSettingsURL() {
   foxyproxy.setSettingsURI(foxyproxy.getDefaultPath());
+  // Writing the current pattern subscriptions back to the default path as well.
+  if (patternSubscriptions.subscriptionsList.length > 0) {
+     patternSubscriptions.writeSubscriptions();
+  } 
   updateSettingsInfo();
-  overlay.alert(this, foxyproxy.getMessage("settings.default"));  
+  overlay.alert(this, foxyproxy.getMessage("settings.default2"));  
 }
 
 /* Contains items which can be updated via toolbar/statusbar/menubar/context-menu as well as the options dialog,
@@ -284,6 +269,7 @@ function _updateView(writeSettings, updateLogView) {
   _updateSuperAdd(foxyproxy.quickadd, "quickAdd");
   document.getElementById("quickAddNotifyWhenCanceled").checked = foxyproxy.quickadd.notifyWhenCanceled; // only exists for QuickAdd
   
+  document.getElementById("toolbarEnabled").checked = foxyproxy.toolbarIcon;
   document.getElementById("toolsMenuEnabled").checked = foxyproxy.toolsMenu;
   document.getElementById("contextMenuEnabled").checked = foxyproxy.contextMenu;
   document.getElementById("statusbarIconEnabled").checked = foxyproxy.statusbar.iconEnabled;
@@ -314,7 +300,8 @@ function _updateView(writeSettings, updateLogView) {
     onQuickAddEnabled(false);
   }
   
-  proxyTree.view = fpc.makeProxyTreeView(foxyproxy, document);
+  proxyTree.view = fpc.makeProxyTreeView(foxyproxy.proxies, document);
+  subscriptionsTree.view = patternSubscriptions.makeSubscriptionsTreeView();
   
   if (writeSettings)
     foxyproxy.writeSettings();
@@ -332,12 +319,21 @@ function onDeleteSelection() {
   if (_isDefaultProxySelected())
     overlay.alert(this, foxyproxy.getMessage("delete.proxy.default"));
   else if (foxyproxy.warnings.showWarningIfDesired(window, ["delete.proxy.confirm"], "confirmDeleteProxy")) {
-	  // Store cur selection
-	  var sel = proxyTree.currentIndex;  
+    // Store cur selection
+    var sel = proxyTree.currentIndex;  
+    // We have to delete the proxy from the subscription as well. Otherwise
+    // there occur errors later on while loading/removing the subscription as
+    // the proxy is still saved in the subscription but not found anymore. We
+    // do this before we delete the proxy itself in order to get the necessary
+    // information (i.e. its id).
+    if (patternSubscriptions.subscriptionsList.length > 0) {
+      let proxyId = foxyproxy.proxies.list[sel].id;
+      patternSubscriptions.removeDeletedProxies(proxyId);
+    } 
     foxyproxy.proxies.remove(proxyTree.currentIndex);
     foxyproxy.broadcast(true /*write settings*/, "foxyproxy-proxy-change");
-	  // Reselect what was previously selected
-		proxyTree.view.selection.select(sel+1>proxyTree.view.rowCount ? 0:sel);    
+    // Reselect what was previously selected
+    proxyTree.view.selection.select(sel+1>proxyTree.view.rowCount ? 0:sel); 
   }  
 }
 
@@ -375,9 +371,9 @@ function onMove(direction) {
 }
 
 function onSettings(isNew) {
-  var sel = proxyTree.currentIndex,
-    params = {inn:{proxy:isNew ?
-      CC["@leahscape.org/foxyproxy/proxy;1"].createInstance().wrappedJSObject : 
+  let sel = proxyTree.currentIndex, selSub = subscriptionsTree.currentIndex,
+    params = {inn:{proxy:isNew ? CC["@leahscape.org/foxyproxy/proxy;1"].
+      createInstance().wrappedJSObject :
       foxyproxy.proxies.item(proxyTree.currentIndex)}, out:null};
         
   window.openDialog("chrome://foxyproxy/content/addeditproxy.xul", "",
@@ -385,8 +381,16 @@ function onSettings(isNew) {
   if (params.out) {
     if (isNew) foxyproxy.proxies.push(params.out.proxy);
     foxyproxy.broadcast(true /*write settings*/, "foxyproxy-proxy-change");
-	  // Reselect what was previously selected or the new item
-		proxyTree.view.selection.select(isNew?proxyTree.view.rowCount-2:sel); 
+    // Reselect what was previously selected or the new item
+    proxyTree.view.selection.select(isNew?proxyTree.view.rowCount-2:sel); 
+    // We need to include this call here as well as the selection is not
+    // clearly visible anymore in the pattern subscription tree without it.
+    // But only redraw the tree if we have one item selected. Otherwise enabling
+    // just "Add New Pattern Subscription" if we have not selected a
+    // subscription does not work properly.
+    if (selSub > -1) {
+      subscriptionsTree.view.selection.select(selSub);
+    }
   }
 }
 
@@ -399,6 +403,146 @@ function setButtons() {
   	(proxyTree.currentIndex+1 < foxyproxy.proxies.length && foxyproxy.proxies.item(proxyTree.currentIndex+1).lastresort));
 }
 
+function addPatternSubscription() {
+  let params = {
+        inn : null,
+        out : null
+      };	
+  window.openDialog('chrome://foxyproxy/content/pattern-subscriptions/addeditsubscription.xul', 
+    '', 'modal, resizable=yes', params).focus(); 
+  if (params.out) {
+    patternSubscriptions.addSubscription(params.out.subscription, 
+      params.out.userValues); 
+    // Now adding the patterns to the proxies provided the user has added
+    // at least one proxy in the addeditsubscription dialog.
+    let proxyList = params.out.proxies; 
+    if (proxyList.length !== 0) {
+      patternSubscriptions.addPatterns(null, proxyList);
+    }
+    subscriptionsTree.view = patternSubscriptions.makeSubscriptionsTreeView();
+  } 
+}
+
+// We need this extra step here. Otherwise the user may click on the empty tree
+// and the "edit-version" of the addeditsubscriptions.xul (Last Status and
+// Refresh not being diabled) would be opened.
+function onDblClickSubscriptionsTree() {
+  if (subscriptionsTree.currentIndex > -1) {
+    editPatternSubscription();
+  }
+}
+
+function editPatternSubscription() {
+  let selectedSubscription = patternSubscriptions.
+    subscriptionsList[subscriptionsTree.currentIndex];
+  let params = {
+        inn : {
+          subscription : selectedSubscription,
+          index : subscriptionsTree.currentIndex
+        }
+      };
+  window.openDialog('chrome://foxyproxy/content/pattern-subscriptions/addeditsubscription.xul', 
+    '', 'modal, resizable=yes', params).focus(); 
+  if (params.out) {
+    patternSubscriptions.editSubscription(selectedSubscription, params.
+      out.userValues, subscriptionsTree.currentIndex);
+    // If new proxies were added we should add the patterns to them as
+    // well but only to them!
+    let proxyList = params.out.proxies; 
+    if (proxyList.length !== 0) {
+      patternSubscriptions.addPatterns(subscriptionsTree.currentIndex,
+        proxyList);
+    } 
+    subscriptionsTree.view = patternSubscriptions.makeSubscriptionsTreeView(); 
+  }
+}
+
+function deletePatternSubscriptions() {
+  // We save the current index to select the proper row after the
+  // subscription got deleted.
+  let selIndex = subscriptionsTree.currentIndex; 
+  // Currently, we have seltype=single that's why "selectecSubscription" is
+  // in singular but it is planned to allow the user to delete more than one
+  // subscription at once. That's why "deletePatternSubscriptions" is in 
+  // plural. The same reasoning holds for the two following functions.
+  let selectedSubscription = patternSubscriptions.subscriptionsList[selIndex];
+  if (foxyproxy.warnings.showWarningIfDesired(window, 
+      ["patternsubscription.del.subscription"], "patSubDelete")) {
+    if (selectedSubscription.timer) {
+      selectedSubscription.timer.cancel();
+    }
+    // Deleting the patterns as well if we have proxies following them...
+    let selSubProxies = selectedSubscription.metadata.proxies;
+    if (selSubProxies.length > 0) {
+      selSubProxies = foxyproxy.proxies.getProxiesFromId(selSubProxies);
+      patternSubscriptions.deletePatterns(selSubProxies, 
+        selectedSubscription.metadata.enabled);
+    }
+    patternSubscriptions.subscriptionsList.splice(selIndex, 1);
+    patternSubscriptions.writeSubscriptions();
+    subscriptionsTree.view = patternSubscriptions.makeSubscriptionsTreeView(); 
+    // Deleting the subscription file if it is empty in order avoid errors
+    // during startup.
+    if (patternSubscriptions.subscriptionsList.length === 0) {
+      dump("Deleting the subscriptions file...\n");
+      patternSubscriptions.getSubscriptionsFile().remove(false);
+      // We need that here otherwise are all options in the context menu still
+      // selected even if no subscription exists anymore.
+      document.getElementById("patsubtree-row-selected").
+        setAttribute("disabled", true);
+    } else {
+      // Easy as we currently have seltype=single
+      if (selIndex === subscriptionsTree.view.rowCount) {
+        selIndex = selIndex - 1;
+      }
+      subscriptionsTree.view.selection.select(selIndex);  
+    }
+  }
+}
+
+function refreshPatternSubscriptions() {
+  patternSubscriptions.refreshSubscription(patternSubscriptions.
+    subscriptionsList[subscriptionsTree.currentIndex], true);
+  subscriptionsTree.view = patternSubscriptions.makeSubscriptionsTreeView();
+}
+
+function viewPatternSubscriptions() {
+  let selectedSubscription = patternSubscriptions.
+    subscriptionsList[subscriptionsTree.currentIndex];
+  let params = {
+        inn : {
+          patterns : selectedSubscription.patterns
+        }
+      };
+  window.openDialog('chrome://foxyproxy/content/pattern-subscriptions/patternsView.xul', 
+    '', 'modal, resizable=yes', params).focus();
+}
+
+function onSubscriptionsAction() {
+  try {
+    switch (document.getElementById("actionList").selectedIndex) {
+      case 0:  
+        addPatternSubscription();
+        break;
+      case 1: 
+        editPatternSubscription();
+        break;
+      case 2:
+        deletePatternSubscriptions();
+        break;
+      case 3:
+        refreshPatternSubscriptions();
+        break;  
+      case 4:
+        viewPatternSubscriptions();
+        break;
+    } 
+  } catch (e) {
+    dump("There went something wrong in the Treeselection: " + e);
+
+  }
+}
+
 function onMaxSize() {
 	var v = document.getElementById("maxSize").value;
 	var passed = true;
@@ -481,12 +625,20 @@ function importSettings() {
     f1.initWithPath(settingsFile.path.substring(0, settingsFile.path.indexOf(settingsFile.leafName)));
     picker.file.copyTo(f1, settingsFile.leafName);
     
-    if (overlay.ask(this, foxyproxy.getMessage("import.success", [picker.file.path])))
+    if (overlay.ask(this, foxyproxy.
+      getMessage("import.success", [picker.file.path]))) {
+      // We have to handle the import and export of pattern subscriptions a bit 
+      // differently here as they are in JSON and not in XML. See as well the 
+      // comment in exportSettings(). "True" and "false" as arguments mean that
+      // we have an import (probably of pattern subscriptions as well) and they 
+      // should be removed from the normal FoxyProxy settings file afterwards.
+      patternSubscriptions.handleImportExport(true, false);
       foxyproxy.restart();
+    }
   }
   catch (e) {
     dump(e + "\n");
-    overlay.alert(this, foxyproxy.getMessage("copy.error", [f1.path, picker.file.path]));
+    overlay.alert(this, foxyproxy.getMessage("copy.error", [picker.file.path, f1.path]));
     return;
   }  
 }
@@ -504,7 +656,17 @@ function importExportPrompt(isExport) {
 function exportSettings() {
   var picker = importExportPrompt(true);
   if (!picker) return;
-
+  // We have the pattern subscriptions and the other FoxyProxy settings in
+  // differernt files in order to reduce the necessary disk I/O while running
+  // FoxyProxy. But we need the pattern subscriptions as well if a user wants
+  // to export her settings. Therefore, handleImportExport() prepares the 
+  // settings file before exporting (i.e. the pattern subscriptions are added) 
+  // if "false" and "true" are passed as parameters. We use "false" and "false" 
+  // as parameters in order to remove the subscriptions from the settings file 
+  // again after it got exported in order not to clutter it unnecessarily.
+  if (patternSubscriptions.subscriptionsList.length > 0) {
+    patternSubscriptions.handleImportExport(false, true);
+  }
   var f2 = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile),
     settingsFile = foxyproxy.getSettingsURI(CI.nsIFile);
   
@@ -521,6 +683,11 @@ function exportSettings() {
     overlay.alert(this, foxyproxy.getMessage("copy.error", [settingsFile.path, picker.file.path]));
     return;
   }
+  finally {
+    if (patternSubscriptions.subscriptionsList.length > 0) {
+      patternSubscriptions.handleImportExport(false, false);
+    } 
+  }
 }
 
 function importProxyList() {
@@ -530,6 +697,12 @@ function onProxyTreeSelected() {
 	setButtons();
 }
 
+function onPatSubTreeSelected() {
+  let selLength = _getSelectedIndices(subscriptionsTree).length; 
+  document.getElementById("patsubtree-row-selected").setAttribute("disabled", 
+    selLength === 0);
+}
+
 function updateLogButtons() {
   document.getElementById("logtree-row-selected").setAttribute("disabled", _getSelectedIndices(logTree).length == 0);
 }
@@ -683,4 +856,19 @@ function copyLogURLToClipboard() {
     if (i+1 != sz) txt += " "; // don't add space to the front or the end
   }
   CC["@mozilla.org/widget/clipboardhelper;1"].getService(CI.nsIClipboardHelper).copyString(txt);
-}
\ No newline at end of file
+}
+
+// We are not animating the icon anmyore due to user complaints but are
+// creating a new icon on every dialog load. Seems to be a good trade-off.
+function createPatternsIcon() {
+  for (let i = 0; i < patternsIcon.length; i++) {
+    let color = Math.round(0xffffff * Math.random()).toString(16); 
+    if (color.length < 6) {
+      do {
+        color = "0" + color;
+      } while (color.length < 6)
+    }
+    patternsIcon[i].setAttribute("style", "fill: " + ("#" + color)
+      + ";fill-opacity:1;fill-rule:nonzero;stroke:none");
+  }
+}
