@@ -6,15 +6,17 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 
 var foxyproxy = {
   fp : Components.classes["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject,
   fpc : Components.classes["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject,
   statusText : null,
-  notes: ["foxyproxy-statusbar-icon","foxyproxy-statusbar-text","foxyproxy-statusbar-width","foxyproxy-toolsmenu",
-    "foxyproxy-contextmenu","foxyproxy-mode-change","foxyproxy-throb","foxyproxy-updateviews","foxyproxy-autoadd-toggle"],
+  notes: ["foxyproxy-toolbarIcon","foxyproxy-statusbar-icon",
+    "foxyproxy-statusbar-text","foxyproxy-statusbar-width",
+    "foxyproxy-toolsmenu","foxyproxy-contextmenu","foxyproxy-mode-change",
+    "foxyproxy-throb","foxyproxy-updateviews","foxyproxy-autoadd-toggle"],
 
   alert : function(wnd, str) {
     Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
@@ -113,6 +115,9 @@ var foxyproxy = {
         this.setMode(str);
         this.checkPageLoad();
         break;
+      case "foxyproxy-toolbarIcon":
+        this.toggleToolbarIcon(e);
+        break;
       case "foxyproxy-toolsmenu":
         this.toggleToolsMenu(e);
         break;
@@ -147,12 +152,13 @@ var foxyproxy = {
   onLoad : function() {
     this.svgIcons.init();
     this.statusText = document.getElementById("foxyproxy-status-text");
-    setTimeout(this.findToolbarIcon, 100);
-    
-    var obSvc = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+    setTimeout(this.defaultToolbarIconFF4, 100);
+    var obSvc = Components.classes["@mozilla.org/observer-service;1"].
+      getService(Components.interfaces.nsIObserverService);
     for (var i in this.notes) {
       obSvc.addObserver(this, this.notes[i], false);
     }
+    this.toggleToolbarIcon(this.fp.toolbarIcon);
     this.toggleToolsMenu(this.fp.toolsMenu);
     this.toggleContextMenu(this.fp.contextMenu);
     this.checkPageLoad();
@@ -171,6 +177,85 @@ end-foxyproxy-simple !*/
       document.getElementById("foxyproxyMenu").setAttribute("label", this.fp.getMessage("foxyproxy.standard.label"));
 /*! end-foxyproxy-standard !*/
     }
+    this.patternErrorNotification();
+  },
+
+  patternErrorNotification : function() {
+    let that = this;
+    let showFailuresOnStartup = {
+      notify: function(timer) {
+        // We show pattern subscription load failures her but only during
+        // startup and not every time a user opens a new top-level window.
+        let winEnum = that.fpc.getEnumerator();
+        for (var winCount = 0; winEnum.hasMoreElements() && winEnum.getNext();
+          winCount++);     
+        if (winCount === 1) {
+          Components.utils.
+            import("resource://foxyproxy/patternSubscriptions.jsm", that); 
+          // Checking whether we had some pattern subscription load failures
+          // during startup (in the first case the whole subscription could not
+          // be loaded and in the second tne just the metadata was available).
+          // If so, we show the proper notification boxes without blocking the
+          // UI using nsITimer.
+          // TODO: Do we really want to have another nsITimer here getting (in
+          // the worst case) multiple warning types?
+          if (that.patternSubscriptions.failureOnStartup) {
+            that.fpc.notify("patternsubscription.error.saved", 
+              [that.patternSubscriptions.failureOnStartup],
+              [{
+                 accessKey: that.fp.getMessage("okay.accesskey"),
+                 callback: function(){},
+                 label: that.fp.getMessage("okay")
+              }], 
+              null, null, false);   
+          }
+          let failedPatternLoad = {
+            notify: function() {
+              for (let i = 0; i < failedSubs.length; i++) {
+                // We got susbcriptions where just the metadata could be loaded.
+                // Asking the user if she wants to refresh the subscription now
+                // in order to have a useable pattern subscription.
+                // We have to do this in a separate method as the current |i|
+                // would otherwise not be preserved and we would always only
+                // refresh the last broken pattern subscription.
+                that.createNotification(that, failedSubs, i);
+              }
+            }
+          }; 
+          let failedSubs = that.patternSubscriptions.partialLoadFailure;
+          if (failedSubs.length > 0) {
+            timer.initWithCallback(failedPatternLoad, 500, 
+	      Components.interfaces.nsITimer.TYPE_ONE_SHOT); 
+          }
+        }  
+      }
+    };
+    Components.classes["@mozilla.org/timer;1"].
+      createInstance(Components.interfaces.nsITimer). 
+      initWithCallback(showFailuresOnStartup, 50, 
+      Components.interfaces.nsITimer.TYPE_ONE_SHOT); 
+  },
+
+  createNotification : function(that, failedSubs, position) {
+    this.fpc.notify("patternsubscription.error.patterns.refresh",
+	          [failedSubs[position].metadata.name], 
+                  [{
+                     accessKey: null,
+                     callback: function() {
+                       that.patternSubscriptions.
+                         refreshSubscription(failedSubs[position], true);
+                     },
+                     label: this.fp.getMessage("yes")
+                  }], 
+                  null, null, false); 
+  },
+
+  toggleToolbarIcon : function(e) {
+    // The code below throws if the icon is not on the toolbar at all. We
+    // therefore catch exceptions.
+    try {
+      document.getElementById("foxyproxy-toolbar-icon").hidden= !e; 
+    } catch(e) {}
   },
 
   toggleToolsMenu : function(e) {
@@ -199,6 +284,10 @@ end-foxyproxy-simple !*/
           foxyproxy.alert(owner, this.fp.getMessage("torwiz.nan"));
           ok = false;
         }
+	if (input.value > 65535 || input.value < 0) {
+          foxyproxy.alert(owner, this.fp.getMessage("torwiz.wrongPort")); 
+	  ok = false;
+	}
       }
       else
         break;
@@ -219,12 +308,12 @@ end-foxyproxy-simple !*/
       p.mode = "manual";
       if (withoutPrivoxy) {
         p.manualconf.host="127.0.0.1";
-        p.manualconf.port=9050;
+        p.manualconf.port=input.value;
         p.manualconf.isSocks=true;
       }
       else {
         p.manualconf.host="127.0.0.1";
-        p.manualconf.port=8118;
+        p.manualconf.port=input.value;
         p.manualconf.isSocks=false;
       }
       p.manualconf.socksversion=5;
@@ -254,6 +343,50 @@ end-foxyproxy-simple !*/
     }
   },
 
+  defaultToolbarIconFF4: function() {
+    foxyproxy.findToolbarIcon(); 
+    let firstRun = foxyproxy.fp.getPrefsService("extensions.foxyproxy.").
+      getBoolPref("firstrun");
+    if (firstRun) {
+      // The Add-on Bar got intruced in FF 4.07b. As it is disabled by default
+      // we show the FoxyProxy toolbar icon on first start in the toolbar to
+      // give the user a hint about FoxyProxy's existence.
+      let vc = Components.classes["@mozilla.org/xpcom/version-comparator;1"].
+        getService(Components.interfaces.nsIVersionComparator); 
+      if (vc.compare(foxyproxy.fpc.appInfo.version, "4.0b7") < 0) {
+        return;
+      }
+      let navBar = document.getElementById("nav-bar"); 
+      if (navBar) {
+        let curSet = navBar.currentSet.split(",");
+        if (curSet.indexOf("foxyproxy-toolbar-icon") === -1) {
+          let pos = curSet.indexOf("urlbar-container") + 1 || curSet.length;
+          let set = curSet.slice(0, pos).concat("foxyproxy-toolbar-icon").
+            concat(curSet.slice(pos));
+          navBar.setAttribute("currentset", set.join(","));
+          navBar.currentSet = set.join(",");
+          document.persist(navBar.id, "currentset");
+           
+          try {
+            BrowserToolboxCustomizeDone(true);
+          } catch (e) {}
+          // We have to duplicate this code here as adding the icon via the
+          // above code does not trigger the customizeChange event in 
+          // findToolbarIcon(). Thus, the proper color would not be applied
+          // to the toolbar icon.
+          foxyproxy.svgIcons.init();
+          if (document.getElementById("fp-toolbar-icon-3")) {
+            // Our toolbar icon was added. Apply the proper icon coloring 
+            // to the toolbar icon by setting the mode again.
+            foxyproxy.setMode(foxyproxy.fp.mode);
+          }
+        } 
+      }
+      foxyproxy.fp.getPrefsService("extensions.foxyproxy.").
+        setBoolPref("firstrun", false); 
+    }
+  },
+
   /**
    * Open or focus the main window/dialog
    */
@@ -571,7 +704,6 @@ end-foxyproxy-simple !*/
   _popupShowing : 0,
 
   onSBTBClick : function(e, o) {
-    e.preventDefault(); // required in Firefox 4 to prevent default context-menu from appearing
     if (e.button==0) {
       _act(o.leftClick, e);
     }
@@ -592,7 +724,16 @@ end-foxyproxy-simple !*/
           break;
         case "contextmenu":
           this._popupShowing = 0;
-          document.getElementById("foxyproxy-statusbar-popup").showPopup(e.target, -1, -1, "popup", "bottomleft", "topleft");
+          let popupElement;
+	  if (e.target.id === "foxyproxy-toolbar-icon") {
+            popupElement = document.
+              getElementById("foxyproxy-toolbarbutton-popup");
+          } else {
+            popupElement = document.
+              getElementById("foxyproxy-statusbar-popup"); 
+          } 
+          popupElement.showPopup(e.target, -1, -1, "popup", "bottomleft",
+            "topleft");
           break;
         case "reloadcurtab":
           gBrowser.reloadTab(gBrowser.mCurrentTab);
@@ -601,14 +742,15 @@ end-foxyproxy-simple !*/
           gBrowser.reloadAllTabs();
           break;
         case "reloadtabsinallbrowsers":
-          for (var b, e = this.fpc.getEnumerator();
-                  e.hasMoreElements();
-                  (b = e.getNext().getBrowser()) && b.reloadAllTabs());
+          for (var b, e = foxyproxy.fpc.getEnumerator();
+            e.hasMoreElements();
+            (b = e.getNext().getBrowser()) && b.reloadAllTabs());
           break;
         case "removeallcookies":
           Components.classes["@mozilla.org/cookiemanager;1"].
             getService(Components.interfaces.nsICookieManager).removeAll();
-          fp.notifier.alert(fp.getMessage("foxyproxy"), fp.getMessage("cookies.allremoved"));
+          fp.notifier.alert(fp.getMessage("foxyproxy"),
+            fp.getMessage("cookies.allremoved"));
           break;
         case "toggle":
           // Toggle between current mode and disabled
@@ -759,6 +901,13 @@ end-foxyproxy-simple !*/
             this.fp.getMessage("foxyproxy.tab.global.accesskey"),
             this.fp.getMessage("foxyproxy.tab.global.tooltip"));
 
+        _createCheckMenuItem(gssubmenupopup,
+          "foxyproxy.fp.toolbarIcon=!foxyproxy.fp.toolbarIcon;foxyproxy.updateViews(false);",
+          this.fp.toolbarIcon,
+          this.fp.getMessage("foxyproxy.toolbaricon.accesskey"),
+          this.fp.getMessage("foxyproxy.toolbaricon.label"),
+          this.fp.getMessage("foxyproxy.toolbaricon.tooltip")); 
+
         _createCheckMenuItem(gssubmenupopup,
           "foxyproxy.fp.statusbar.iconEnabled=!foxyproxy.fp.statusbar.iconEnabled;foxyproxy.updateViews(false);",
           this.fp.statusbar.iconEnabled,
@@ -777,8 +926,8 @@ end-foxyproxy-simple !*/
           "foxyproxy.fp.toolsMenu=!foxyproxy.fp.toolsMenu;foxyproxy.updateViews(false);",
           this.fp.toolsMenu,
           this.fp.getMessage("foxyproxy.toolsmenu.accesskey"),
-          this.fp.getMessage("foxyproxy.toolsmenu.label"),
-          this.fp.getMessage("foxyproxy.toolsmenu.tooltip"));
+          this.fp.getMessage("foxyproxy.toolsmenu.label2"),
+          this.fp.getMessage("foxyproxy.toolsmenu.tooltip2"));
 
         _createCheckMenuItem(gssubmenupopup,
           "foxyproxy.fp.contextMenu=!foxyproxy.fp.contextMenu;foxyproxy.updateViews(false);",
@@ -825,13 +974,13 @@ end-foxyproxy-simple !*/
              this.fp.getMessage("foxyproxy.refresh.tooltip"));
   
           itm =_createMenuItem(submenupopup,
-              this.fp.getMessage("foxyproxy.quickadd.label"),
-              "foxyproxy.onQuickAddDialog(event)",
-              this.fp.getMessage("foxyproxy.quickadd.accesskey"),
-              this.fp.getMessage("foxyproxy.quickadd.tooltip"));
-            itm.setAttribute("key", "key_foxyproxyquickadd");
-            itm.setAttribute("disabled", disableQuickAdd(this.fp));          
-  
+            this.fp.getMessage("foxyproxy.quickadd.label"),
+            "foxyproxy.onQuickAddDialog(event)",
+            this.fp.getMessage("foxyproxy.quickadd.accesskey"),
+            this.fp.getMessage("foxyproxy.quickadd.tooltip"));
+          itm.setAttribute("key", "key_foxyproxyquickadd");
+          itm.setAttribute("disabled", disableQuickAdd(this.fp));
+
           _createCheckMenuItem(logsubmenupopup,
             // no need to write settings because changing the attribute makes the fp service re-writes the settings
             "foxyproxy.onToggleNoURLs();",
@@ -849,17 +998,6 @@ end-foxyproxy-simple !*/
           this.fp.getMessage("foxyproxy.options.tooltip"));
         itm.setAttribute("key", "key_foxyproxyfocus");
 
-        if (!isFoxyProxySimple) {
-          // No quickadd for FoxyProxy Simple
-          itm =_createMenuItem(submenupopup,
-            this.fp.getMessage("foxyproxy.quickadd.label"),
-            "foxyproxy.onQuickAddDialog(event)",
-            this.fp.getMessage("foxyproxy.quickadd.accesskey"),
-            this.fp.getMessage("foxyproxy.quickadd.tooltip"));
-          itm.setAttribute("key", "key_foxyproxyquickadd");
-          itm.setAttribute("disabled", disableQuickAdd(this.fp));
-        }
-                  
         _createMenuItem(submenupopup,
           this.fp.getMessage("foxyproxy.help.label"),
           "foxyproxy.fpc.openAndReuseOneTabPerURL('http://getfoxyproxy.org/" + (isFoxyProxySimple ? "basic/" : "") + "help.html');",
