@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 var exampleURL, pattern, generatedPattern, caseSensitive, fpc, isSuperAdd;
 function onLoad() {
@@ -24,7 +24,7 @@ function onLoad() {
     document.getElementById("not-superadd").setAttribute("hidden", true);
   }
   else {
-    enabled = m.enabled;
+    var enabled = m.enabled;
     document.getElementById("superadd").setAttribute("hidden", true);
     document.getElementById("not-superadd").setAttribute("hidden", false);
   }
@@ -56,4 +56,4 @@ function onOK() {
 
 function updateGeneratedPattern() {
   generatedPattern.value = fpc.applyTemplate(exampleURL.value, pattern.value, caseSensitive.checked);
-}
\ No newline at end of file
+}
