@@ -6,7 +6,7 @@
 
   This source code is released under the GPL license,
   available in the LICENSE file at the root of this installation
-  and also online at http://www.gnu.org/licenses/gpl.txt
+  and also online at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 **/
 
 /**
@@ -17,7 +17,7 @@ foxyproxy.selection = {
   onChangeHost : function() {
     var fp = foxyproxy.fp, sel = this.parseSelection();
     if (sel.reason == 0) {
-      var p = {inn:{title:fp.getMessage("choose.proxy", [sel.hostPort]), reloadcurtab:this.reloadcurtab, host:sel.host, port:sel.port}, out:null};          
+      var p = {inn:{title:fp.getMessage("choose.proxy", [sel.hostPort]), reloadcurtab:this.reloadcurtab, host:sel.host, port:sel.port, pattern:false}, out:null};          
       window.openDialog("chrome://foxyproxy/content/chooseproxy.xul", "",
         "chrome, dialog, modal, centerscreen=yes, resizable=yes", p).focus();
       if (p.out) {
@@ -141,4 +141,4 @@ foxyproxy.selection = {
     return selection;
   }
 
-};
\ No newline at end of file
+};
